<?php
/*******************************************************************************
*    myMVC - Model-View-Controller for PHP                                     *
*    Copyright (C) 2012 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU Lesser General Public License as published  *
*    by the Free Software Foundation; version 2.1 of the License.              *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU Lesser General Public License for more details.                       *
*                                                                              *
*    You should have received a copy of the GNU Lesser General Public License  *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

/**
 * Database connector for MySQL (http://www.mysql.com/)
 */
class myMVC_DatabaseConnectorMysql extends myMVC_DatabaseConnector
{
  protected static $connection;
  protected static $usedDatabase;

  /**
   * Wrapper for the errno-function of the selected database engine
   * 
   * @return int                the error number
   */
  protected function errno()
  {
    return @mysqli_errno(self::$connection);
  }

  /**
   * Wrapper for the error-function of the selected database engine
   * 
   * @return string             the error string
   */
  protected function error()
  {
    return @mysqli_error(self::$connection);
  }

  /**
   * Connects to the database.
   * This will just be done, if no connection exists yet.
   *  
   * @return resource           a connection resource
   * @throws myMVC_DatabaseException        if no connection is possible
   */ 
  protected function getDatabaseConnection()
  {
    global $config;

    if($this->database != self::$usedDatabase && self::$usedDatabase !== NULL)
    {
      self::$connection = NULL;
      myMVC_Logger::doLog(__METHOD__, __LINE__, 'Database name changed. Resetting connection', ALERT_WARNING);
    }
    if(self::$connection === NULL || self::$connection == 0)
    {
      self::$connection = @mysqli_connect($this->server, $this->login, $this->password, false);
      if(self::$connection === FALSE)
      {
        self::$connection = NULL;
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'no connection '.$this->errno().': '.$this->error());
        throw new myMVC_DatabaseException($msg);
      }
      self::$usedDatabase = $this->database;
      if(!@mysqli_select_db(self::$connection, $this->database))
      {
        self::$connection = NULL;
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'cannot select db '.$this->errno().': '.$this->error());
        throw new myMVC_DatabaseException($msg);
      }
      if(!@mysqli_query(self::$connection, "SET NAMES utf8"))
      {
        self::$connection = NULL;
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'cannot set connection to unicode '.$this->errno().': '.$this->error());
        throw new myMVC_DatabaseException($msg);
      }
    }

    return self::$connection;
  }

  /**
   * Quotes a string to use in a SQL-Query (with LIKE)
   * 
   * @param  string $value      The string with the Query
   * @param  string $prefix     (normally %) added BEFORE the string
   * @param  string $suffix     (normally %) added AFTER the string
   * @return string             A secure SQL-String (quoted)
   */
  protected function quoteSqlStringForLike($value, $prefix = '%', $suffix = '%')
  {
    if(get_magic_quotes_gpc())
      $value = stripslashes($value);

    if(is_numeric($value))
    {
      $value = "'$prefix".$value."$suffix'";
    }
    else
    {
      try
      {
        $value = "'$prefix" . @mysqli_real_escape_string($this->getDatabaseConnection(), $value) . "$suffix'";
      }
      catch(myMVC_DatabaseException $e)
      {
        ErrorHandler::systemError($e);
        return '';
      }
    }   
    return $value;
  }

  /**
   * Execute an INSERT, UPDATE or DELETE query
   * 
   * @param  string $query      The SQL-Query
   * @return boolean            true on success, false on failure, last inserted ID on successfully INSERT
   */
  protected function execute($query)
  {
    global $config;

    myMVC_Logger::doLog(__METHOD__, __LINE__, "Query: $query ", ALERT_SQL);

    try
    { 
      if(@mysqli_query($this->getDatabaseConnection(), $query) === false)
      {
        myMVC_Logger::doLog(__METHOD__, __LINE__, $this->errno().': '.$this->error()."\n SQL: $query", ALERT_WARNING, true);
        return false;
      }
      if(strncasecmp($query, "INSERT", 6) == 0)
        return @mysqli_insert_id($this->getDatabaseConnection());
    }
    catch (myMVC_DatabaseException $e)
    {
      ErrorHandler::systemError($e);
      return false;
    }     
    return true;
  }

  /**
   * Executes a SELECT-Query
   * 
   * @param  string $query      The SQL-Query
   * @return resource           a resource to get data on success, false otherwise
   */
  protected function executeSelect($query)
  {
    global $config;

    myMVC_Logger::doLog(__METHOD__, __LINE__, "Query: $query ", ALERT_SQL);

    try
    { 
      if(($result = @mysqli_query($this->getDatabaseConnection(), $query)) === false)
      {
        myMVC_Logger::doLog(__METHOD__, __LINE__, $this->errno().': '.$this->error()."\n SQL: $query", ALERT_WARNING, true);
        return false;
      }
      return $result;
    }
    catch (myMVC_DatabaseException $e)
    {
      ErrorHandler::systemError($e);
      return false;
    }     
  }

  /**
   * Returns a record as associative array, or NULL if no record can be returned.
   * 
   * @param resource $result    The resource
   * @return array              The record
   */
  protected function getValueArrayFromResult($result)
  {
    $row = @mysqli_fetch_assoc($result);
    if($row === false)
      return NULL;
    return $row;
  }

  /**
   * Returns a record as numeric array, or NULL if no record can be returned.
   * 
   * @param resource $result    The resource
   * @return array              The record
   */
  protected function getValuesFromResult($result)
  {
    $row = @mysqli_fetch_array($result, MYSQLI_NUM);
    if($row === false)
      return NULL;
    return $row;
  }

  /**
   * Returns the number of row of the runned query.
   * 
   * @param resource $result    The resource
   * @return int                The row's number
   */
  protected function getRowsNum($result)
  {
    return @mysqli_num_rows($result);
  }

  /**
   * Free the allocated memory for the SQL query.
   * 
   * @param resource $result    The resource
   * @return boolean            true on success, false on problem
   */
  protected function freeResult($result)
  {
    return @mysqli_free_result($result);
  }
}
?>
