<?php
/*******************************************************************************
*    myMVC - Model-View-Controller for PHP                                     *
*    Copyright (C) 2012 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU Lesser General Public License as published  *
*    by the Free Software Foundation; version 2.1 of the License.              *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU Lesser General Public License for more details.                       *
*                                                                              *
*    You should have received a copy of the GNU Lesser General Public License  *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

/**
 * Class to manage the sessions.
 * It supports normal session (managed from PHP) or
 * database ones.
 * Session-ID can be as URL-parameter or as Cookie.
 * See mymvc.php for configuration items
 */
class myMVC_HttpSession
{
  private $currentNamespace;
  private static $instance;
  private static $dbEngine;
  private $sessionData;

  /**
   * Constructor.
   * WARNING: This class has to be used as Singleton!
   * Example:
   * 
   * <code>
   * $session = myMVC_HttpSession::getInstance();
   * </code>
   */
  private function __construct()
  {
    global $config;

    $this->sessionData = array();
    $this->startNewSession();
    if($config['SESSION_TYPE'] == DB_SESSION)
    {
      register_shutdown_function(array($this, 'shutdown'));
      $this->dbEngine = new myMVC_SessionDB();
      if(!$_SESSION['__private']['DB_DATA'])
      {
        if(!is_array($config['SESSION_DB']))
        {
          $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, "\$config['SESSION_DB'] is not an array");
          throw new myMVC_SessionException($msg);
        }
        $cfg = $config['SESSION_DB'];
        if(!is_string($cfg['table']) ||
           !is_string($cfg['field_sid']) ||
           !is_string($cfg['field_data']) ||
           !is_string($cfg['field_last_used']))
        {
          $msg = myMVC_Logger::doLog(__METHOD__, __LINE__,
              "You MUST define 'table', 'field_sid', 'field_data' and 'field_last_used' in \$config['SESSION_DB']'");
          throw new myMVC_SessionException($msg);
        }
        $_SESSION['__private']['DB_DATA'] = true;
      }
      $this->sessionData = $this->dbEngine->getSessionDataFromDB($this->getSessionId());
    }
    else
    {
      if(isset($_SESSION['__sid_data']))
        $this->sessionData = $_SESSION['__sid_data'];
      else
        $this->sessionData = $_SESSION['__sid_data'] = array();
    }
  }

  /**
   * Destructor.
   * Saves the data in the session or the database, depending on
   * the value of $config['SESSION_TYPE'] (see mymvc.php)
   */
  public function __destruct()
  {
    global $config;

    if($config['SESSION_TYPE'] == DB_SESSION)
    {
      $this->dbEngine->saveSessionDataIntoDB($this->getSessionId(), $this->sessionData);
      $this->dbEngine->deleteOldSessions();
    }
    else
      $_SESSION['__sid_data'] = $this->sessionData;
  }

  /**
   * Called from PHP if the script terminates with exit()
   */
  public function shutdown()
  {
    myMVC_Logger::doLog(__METHOD__, __LINE__, 'Script terminates unexpectly. Saving DB session', ALERT_WARNING);
    $this->dbEngine->saveSessionDataIntoDB($this->getSessionId(), $this->sessionData);
    $this->dbEngine->deleteOldSessions();
  }

  /**
   * Returns the instance of this class.
   * 
   * @return myMVC_HttpSession      An instance of the session class
   */
  public static function getInstance()
  { 
    if(!isset(self::$instance))
    { 
      $c = __CLASS__; 
      self::$instance = new $c(); 
    } 
    return self::$instance; 
  }

  /**
   * Starts a new session.
   * Lifetime in seconds and type (URL or Cookies) can be set in the configuration
   * ($config['SESSION_LIFETIME'] and $config['SESSION_USECOOKIES']).
   * Default are 8 hours for lifetime and URL for session
   */
  protected function startNewSession()
  {
    global $config;

    $sapi_type = php_sapi_name();
    if(strstr($sapi_type, 'cli') === false)
    {
      ini_set("session.auto_start", "0");
      ini_set("session.use_cookies", $config['SESSION_USECOOKIES']);
      ini_set("session.gc_maxlifetime", $config['SESSION_LIFETIME']);
      session_cache_expire($config['SESSION_LIFETIME']);
      session_name('sid');
      session_start();
    }
    $this->nameSpace = '';
  }

  /**
   * Returns if the current session is expired
   * 
   * @return boolean                true if the session is expired,
   *                                false if it's active
   */
  public function isExpired()
  {
    $status = false;
    if(version_compare(phpversion(), '5.4.0', '>='))
      $status = (session_status() == PHP_SESSION_ACTIVE);
    else
      $status = (session_id() != '');
    return !$status || !key_exists('__sid_data', $_SESSION);
  }

  /**
   * Returns the current session ID
   * 
   * @return string                 The SessionID
   */
  public function getSessionId()
  {
    return SID;
  }

  /**
   * Destroy the current Session (for example due to a logout)
   * and starts a new one.
   */
  public function destroy()
  {
    session_unset();
    session_destroy();
    session_write_close();
    $this->sessionData = array();
    myMVC_Logger::doLog(__METHOD__, __LINE__, 'Destroying session', ALERT_DEBUG);
    $this->startNewSession();
  }

  /**
   * Set the namespace to use
   * 
   * @param string $namespace       The namespace to be used
   */
  public function useNamespace($namespace)
  {
    $this->currentNamespace = $namespace;
  }

  /**
   * Empty the current used namespace
   */
  public function emptyNamespace()
  {
    unset($this->sessionData[$this->currentNamespace]);
  }

  /**
   * Empty a given namespace
   * 
   * @param string $namespace       Namespace to be deleted
   */
  public function emptyGivenNamespace($namespace)
  {
    unset($this->sessionData[$namespace]);
  }

  /**
   * Save an object in the current namespace
   * 
   * @param string $key             Name of the variable
   * @param object $value           Object to be saved
   */
  public function set($key, $value)
  {
    $this->sessionData[$this->currentNamespace][$key] = $value;
  }

  /**
   * Save an object in the given namespace
   * 
   * @param string $namespace       Namespace to be used for saving
   * @param string $key             Name of the variable
   * @param object $value           Object to be saved
   */
  public function setInNamespace($namespace, $key, $value)
  {
    $this->sessionData[$namespace][$key] = $value;
  }

  /**
   * Delete an object from current namespace
   * 
   * @param string $key             Name of the variable
   */
  public function delete($key)
  {
    unset($this->sessionData[$this->currentNamespace][$key]);
  }

  /**
   * Delete an object from the given namespace
   * 
   * @param string $namespace       Namespace to be used for deleting
   * @param string $key             Name of the variable
   */
  public function deleteFromNamespace($namespace, $key)
  {
    unset($this->sessionData[$namespace][$key]);
  }

  /**
   * Returns a previously saved object in the current namespace
   * 
   * @param string $key             Name of the variable
   * @return object                 Value
   */
  public function get($key)
  {
    if(isset($this->sessionData[$this->currentNamespace][$key]))
      return $this->sessionData[$this->currentNamespace][$key];
    return NULL;
  }

  /**
   * Returns a previously saved object in the given namespace
   * 
   * @param string $namespace       Namespace to be used for get
   * @param string $key             Name of the variable
   * @return object                 Value
   */
  public function getFromNamespace($namespace, $key)
  {
    if(isset($this->sessionData[$namespace][$key]))
      return $this->sessionData[$namespace][$key];
    return NULL;
  }

  /**
   * Returns whether an object exists in the current namespace
   * 
   * @param string $key             Name of the variable
   * @return boolean                true if exists, false otherwise
   */
  public function exists($key)
  {
    return isset($this->sessionData[$this->currentNamespace][$key]);
  }

  /**
   * Returns whether an object exists in the given namespace
   * 
   * @param string $namespace       Namespace to be used for get
   * @param string $key             Name of the variable
   * @return boolean                true if exists, false otherwise
   */
  public function existsInNamespace($namespace, $key)
  {
    return isset($this->sessionData[$namespace][$key]);
  }
}
?>
