<?php
/*******************************************************************************
*    myMVC - Model-View-Controller for PHP                                     *
*    Copyright (C) 2012 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU Lesser General Public License as published  *
*    by the Free Software Foundation; version 2.1 of the License.              *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU Lesser General Public License for more details.                       *
*                                                                              *
*    You should have received a copy of the GNU Lesser General Public License  *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/
(include_once('Mail.php')) or die("PEAR-Package 'Mail' is not installed");
(include_once('Mail/mime.php')) or die("PEAR-Package 'Mail/mime' is not installed");

/**
 * Message management.
 * With this class it is possible to send E-Mails
 */
class myMVC_MessageManager
{
  /**
   * Send an E-Mail to the given address.
   * It is possible to pass the engine some variables to substitute placeholders.
   * 
   * @param Class $mailTemplate         Template of the E-Mail. It MUST derive myMVC_BaseView
   * @param boolean $isHTML             if true, the E-Mail will send the template as HTML, too. Otherwise is the E-Mail plain-text
   * @param array $variables            Optional template variables as associative array (name => value)
   * @param string $from                Sender's address
   * @param string $to                  Recipient's address (for the E-Mail-headers)
   * @param string $subject             Subject of the E-Mail
   * @param array $recipients           Recipients' list. Hier you can give more addresses as in $to
   * @param array $extraHeaders         Optionally extra headers as associative array header => value
   * @param string $smtpServer          If given, myMVC uses this server to send the E-Mail. Otherwise it uses the server defined in $config
   * @param string $smtpAuth            If given, myMVC uses this authentification method to send the E-Mail. Otherwise it uses the server defined in $config
   * @param string $smtpUser            If given, myMVC uses this login to send the E-Mail. Otherwise it uses the server defined in $config
   * @param string $smtpPass            If given, myMVC uses this password to send the E-Mail. Otherwise it uses the server defined in $config
   * @param array $attachments          If given, list of files to be sent as attachment
   *                                    (array of associative arrays. Key: "file" => filename, "ctype" => content type, "displayname" => display name, "type" => "string" or "file")
   * @return mixed                      true if E-Mail sent, PEAR_Error on problem
   */
  public static function sendEMail($mailTemplate, $isHTML, $variables, $from, $to, $subject, $recipients, $extraHeaders = array(), $smtpServer = NULL, $smtpAuth = NULL, $smtpUser = NULL, $smtpPass = NULL, $attachment = array())
  {
    global $config;

    if(!is_subclass_of($mailTemplate, 'myMVC_BaseView'))
    {
      $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Template class %s is not a subclass of myMVC_BaseView", get_class($mailTemplate)));
      throw new myMVC_MVCException($msg);
    }

    foreach(array_keys($variables) as $key)
      $mailTemplate->setVariable($key, $variables[$key]);

    $mailTemplate->preRender();
    return self::sendPlainEMail($mailTemplate->fetch(), $isHTML, $from, $to, $subject, $recipients, $extraHeaders, $smtpServer, $smtpAuth, $smtpUser, $smtpPass, $attachment);
  }

  /**
   * Send an E-Mail to the given address
   * 
   * @param string $mailTemplate        Template of the E-Mail as string
   * @param boolean $isHTML             if true, the E-Mail will send the template as HTML, too. Otherwise is the E-Mail plain-text
   * @param string $from                Sender's address
   * @param string $to                  Recipient's address (for the E-Mail-headers)
   * @param string $subject             Subject of the E-Mail
   * @param array $recipients           Recipients' list. Hier you can give more addresses as in $to
   * @param array $extraHeaders         Optionally extra headers as associative array header => value
   * @param string $smtpServer          If given, myMVC uses this server to send the E-Mail. Otherwise it uses the server defined in $config
   * @param string $smtpAuth            If given, myMVC uses this authentification method to send the E-Mail. Otherwise it uses the server defined in $config
   * @param string $smtpUser            If given, myMVC uses this login to send the E-Mail. Otherwise it uses the server defined in $config
   * @param string $smtpPass            If given, myMVC uses this password to send the E-Mail. Otherwise it uses the server defined in $config
   * @param array $attachments          If given, list of files to be sent as attachment
   *                                    (array of associative arrays. Key: "file" => filename, "ctype" => content type, "displayname" => display name, "type" => "string" or "file")
   * @return mixed                      true if E-Mail sent, PEAR_Error on problem
   */
  public static function sendPlainEMail($mailTemplate, $isHTML, $from, $to, $subject, $recipients, $extraHeaders = array(), $smtpServer = NULL, $smtpAuth = NULL, $smtpUser = NULL, $smtpPass = NULL, $attachment = array())
  {
    global $config;

    if($smtpServer !== NULL)
    {
      if($smtpAuth === NULL)
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'SMTP-Server given, but auth is empty');
        throw new myMVC_MVCException($msg);
      }
      if($smtpUser === NULL)
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'SMTP-Server given, but login is empty');
        throw new myMVC_MVCException($msg);
      }
      if($smtpPass === NULL)
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'SMTP-Server given, but password is empty');
        throw new myMVC_MVCException($msg);
      }
    }
    else
    {
      $smtpServer = $config['MAIL']['SMTP'];
      $smtpAuth = $config['MAIL']['AUTH'];
      $smtpUser = $config['MAIL']['USER'];
      $smtpPass = $config['MAIL']['PASS'];
    }

    if($isHTML)
    {
      // Strips inline CSS
      $startCSS = strpos($mailTemplate, '<style');
      $endCSS = strpos($mailTemplate, '</style>');
      $mailTemplate = substr($mailTemplate, 0, $startCSS).substr($mailTemplate, $endCSS + strlen('</style>'));
    }

    // Conversion of special characters (german umlauts, accented vocals)
    $mailTexts = str_replace(
            array('&auml;', '&ouml;', '&uuml;', '&Auml;', '&Ouml;', '&Uuml;', '&szlig;',
                  '&aacute', '&agrave', '&eacute', '&egrave', '&iacute', '&igrave', '&oacute', '&ograve', '&uacute', '&ugrave',
                  '&Aacute', '&Agrave', '&Eacute', '&Egrave', '&Iacute', '&Igrave', '&Oacute', '&Ograve', '&Uacute', '&Ugrave'),
            array('ä', 'ö', 'ü', 'Ä', 'Ö', 'Ü', 'ß',
                  'á', 'à', 'é', 'è', 'í', 'ì', 'ó', 'ò', 'ú', 'ù',
                  'Á', 'À', 'É', 'È', 'Í', 'Ì', 'Ó', 'Ò', 'Ú', 'Ù'),
            $mailTemplate);
    $mailTexts = explode("\n", strip_tags($mailTexts));
    $mailText = '';
    foreach($mailTexts as $t)
      $mailText .= ltrim($t)."\n";

    $hdrs = array(
            'From' => $from,
            'To' => $to,
            'Subject' => $subject,
            'Message-Id' => sprintf('<%s.%d@%s>', date('YmdHis'), $config['myPID'], $config['nodeName']),
            'Date' => date('r'),
            'X-Loop' => 'yes',
            'Precedence' => 'bulk',
            'Auto-Submitted' => 'auto-generated'
            );
    foreach(array_keys($extraHeaders) as $h)
      $hdrs[$h] = $extraHeaders[$h];
    if(is_array($recipients))
      $toSendRecipients = $recipients;
    else
      $toSendRecipients = array($recipients);
    $mime = new Mail_mime("\n");
    $mime->setTXTBody($mailText);
    if($isHTML)
      $mime->setHTMLBody($mailTemplate);
    foreach($attachment as $att)
    {
      if($mime->addAttachment($att['file'], $att['ctype'], $att['displayname'], ($att['type'] == 'file')) !== true)
        return false;
    }
    if($isHTML)
      $body = $mime->get(array(
                              "text_charset"    => "UTF-8",
                              "html_charset"    => "UTF-8",
                              "head_charset"    => "UTF-8"));
    else 
      $body = $mime->get(array(
                              "text_charset"    => "UTF-8",
                              "head_charset"    => "UTF-8"));
    $hdrs = $mime->headers($hdrs);
    $mail =& Mail::factory('smtp',
                            array('host' => $smtpServer,
                                  'auth' => $smtpAuth,
                                  'username' => $smtpUser,
                                  'password' => $smtpPass,
                                  'localhost' => $config['nodeName']));
    return $mail->send($toSendRecipients, $hdrs, $body);
  }
}
?>
